package com.bipt.intelligentapplicationorchestrationservice.gpu.cache;

import com.bipt.intelligentapplicationorchestrationservice.gpu.dao.GpuResourceDao;
import com.bipt.intelligentapplicationorchestrationservice.gpu.exception.CacheInitException;
import com.bipt.intelligentapplicationorchestrationservice.gpu.model.entity.GpuResource;
import jakarta.annotation.PostConstruct;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.data.redis.RedisConnectionFailureException;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Component;

import java.time.LocalDateTime;
import java.util.List;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.locks.ReentrantLock;

@Component
public class CacheManager {
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;

    @Autowired
    private GpuResourceDao gpuResourceDao;

    private final ReentrantLock lock = new ReentrantLock();

    @Value("${cache.redis-key-prefix:gpu:}")
    private String keyPrefix;

    @Value("${cache.ttl-base:7200}")
    private int ttlBase;

    @Value("${cache.init-batch-size:500}")
    private int initBatchSize;

    // 全量加载（带分页和分布式锁）
    @PostConstruct
    public void loadFullCache() {
        if (tryLock()) {
            try {
                int page = 0;
                while (true) {
                    List<GpuResource> batch = gpuResourceDao.findByPage(page * initBatchSize, initBatchSize);
                    if (batch.isEmpty()) break;

                    batch.forEach(this::setCacheWithTTL);
                    page++;
                }
            } finally {
                unlock();
            }
        }
    }

    // 单条缓存刷新（带版本控制）
    public void refreshCache(Long gpuId) {
        GpuResource latest = gpuResourceDao.selectByIdWithLock(gpuId);
        if (latest != null) {
            setCacheWithTTL(latest);
        }
    }

    // 批量增量同步
    public void syncCache(LocalDateTime lastSyncTime) {
        List<GpuResource> updates = gpuResourceDao.findModifiedSince(lastSyncTime);
        updates.forEach(entity -> {
            if (entity.getIsDeleted()) {
                redisTemplate.delete(buildKey(entity.getGPUId().toString()));
            } else {
                setCacheWithTTL(entity);
            }
        });
    }

    // 带随机TTL的缓存设置
    private void setCacheWithTTL(GpuResource entity) {
        String key = buildKey(entity.getGPUId().toString());
        redisTemplate.opsForValue().set(
                key,
                entity,
                ttlBase + (int)(Math.random() * 600), // 随机TTL防止雪崩
                TimeUnit.SECONDS
        );
    }

    // 构建缓存键
    private String buildKey(String gpuId) {
        return keyPrefix + gpuId;
    }

    // 分布式锁操作
    private boolean tryLock() {
        try {
            return lock.tryLock(30, TimeUnit.SECONDS);
        } catch (InterruptedException e) {
            Thread.currentThread().interrupt();
            return false;
        }
    }

    private void unlock() {
        lock.unlock();
    }
    // 分页加载入口
    public void loadFullCache(int batchSize) {
        int page = 0;
        while (true) {
            List<GpuResource> batch = gpuResourceDao.findByPage(page * batchSize, batchSize);
            if (batch.isEmpty()) break;

            batch.forEach(this::refreshWithRetry); // 带重试的刷新逻辑
            page++;
        }
    }

    // 带重试机制的缓存刷新
    private void refreshWithRetry(GpuResource entity) {
        try {
            setCacheWithTTL(entity);
        } catch (RedisConnectionFailureException ex) {
            // 3次重试逻辑
            for (int i = 0; i < 3; i++) {
                try {
                    Thread.sleep(1000);
                    setCacheWithTTL(entity);
                    return;
                } catch (InterruptedException e) {
                    Thread.currentThread().interrupt();
                }
            }
            throw new CacheInitException("缓存刷新失败: " + entity.getGPUId().toString());
        }
    }

    // 获取当前缓存数量（调试用）
    public long getCacheCount() {
        return redisTemplate.keys(keyPrefix + "*").size();
    }

    public void evictCache(Long gpuId) {
        String key = buildKey(gpuId.toString());
        redisTemplate.delete(key);
    }
}
