package com.bipt.intelligentapplicationorchestrationservice.gpu.cache.task;

import com.bipt.intelligentapplicationorchestrationservice.gpu.cache.CacheManager;
import com.bipt.intelligentapplicationorchestrationservice.gpu.dao.GpuResourceDao;
import com.bipt.intelligentapplicationorchestrationservice.gpu.model.entity.GpuResource;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;
import java.time.LocalDateTime;
import java.util.List;

//@Slf4j
@Component
public class CacheSyncTask {
    private final GpuResourceDao gpuResourceDao;
    private final CacheManager cacheManager;
    private LocalDateTime lastSyncTime = LocalDateTime.MIN;
    private static final Logger log = LoggerFactory.getLogger(CacheSyncTask.class);

    @Autowired
    public CacheSyncTask(GpuResourceDao gpuResourceDao, CacheManager cacheManager) {
        this.gpuResourceDao = gpuResourceDao;
        this.cacheManager = cacheManager;
    }

    /**
     * 定时同步缓存（默认每10分钟）
     */
    @Scheduled(fixedDelayString = "${cache.sync-interval:600000}")
    public void syncCache() {
        try {
            LocalDateTime currentSyncTime = LocalDateTime.now();
            log.info("🔄 开始缓存同步 | 时间范围: {} - {}", lastSyncTime, currentSyncTime);

            // 1. 查询增量数据
            List<GpuResource> modifiedGpus = gpuResourceDao.findModifiedSince(lastSyncTime);
            if (modifiedGpus.isEmpty()) {
                log.info("✅ 无数据变更，跳过本次同步");
                return;
            }

            // 2. 处理数据变更
            modifiedGpus.forEach(gpu -> {
                if (gpu.getIsDeleted()) {
                    cacheManager.evictCache(gpu.getGPUId());
                    log.debug("🗑️ 删除缓存 | GPU ID: {}", gpu.getGPUId());
                } else {
                    cacheManager.refreshCache(gpu.getGPUId());
                    log.debug("🔄 更新缓存 | GPU ID: {}", gpu.getGPUId());
                }
            });

            // 3. 更新同步时间戳
            lastSyncTime = currentSyncTime;
            log.info("✅ 缓存同步完成 | 共处理 {} 条记录", modifiedGpus.size());

        } catch (Exception e) {
            log.error("❌ 缓存同步失败: {}", e.getMessage(), e);
        }
    }
}
